--
-- PFLISTS
--
-- ruleset script 
-- this script generates, installs and activates a PF ruleset
-- using data from pflists database and options
--

-- PROPERTIES
property binaryPath : missing value -- path to running application bundle
property macosxVersion : missing value -- os x version
property appdir : "/Library/Preferences/PFLists/" -- PFLists configuration working directory (ruleset database)
property confFile : "/Library/PFLists/pflists.conf"
property tablesFile : "/Library/PFLists/pflists.tables"
property oldStatus : missing value
property etURL : "http://rules.emergingthreats.net/fwrules/emerging-Block-IPs.txt"

-- CLICKED
on clicked theObject
	if the name of theObject is equal to "apply" then
		set the visible of image view "picerr" of tab view item "firewall" of tab view "main" of window "main" to false as boolean
		set the visible of image view "picmid" of tab view item "firewall" of tab view "main" of window "main" to true as boolean
		set oldStatus to the contents of text field "bannerfirewall" of tab view item "firewall" of tab view "main" of window "main"
		set the contents of text field "bannerfirewall" of tab view item "firewall" of tab view "main" of window "main" to "Apply changes?"
		tell progress indicator "spinner" of window "main" to start
		display panel window "confirmapply" attached to window "main"
	else if the name of theObject is equal to "doapply" then
		close panel window "confirmapply"
		applyRuleset()
		--displayDialogOK("Done.")
	else if the name of theObject is equal to "installscripts" then
		installScripts()
	else if the name of theObject is equal to "uninstallscripts" then
		uninstallScripts()
	else if the name of theObject is equal to "restartpf" then
		PFrestart()
	else if the name of theObject is equal to "pfstop" then
		PFstop()
	else if the name of theObject is equal to "uninstallpflists" then
		uninstallEverything()
	else if the name of theObject is equal to "startstop" then
		set the visible of image view "picerr" of tab view item "firewall" of tab view "main" of window "main" to false as boolean
		set the visible of image view "picmid" of tab view item "firewall" of tab view "main" of window "main" to true as boolean
		if the title of button "startstop" of tab view item "firewall" of tab view "main" of window "main" is equal to "Start" then
			set the contents of text field "bannerfirewall" of tab view item "firewall" of tab view "main" of window "main" to "Start PF firewall?"
			tell progress indicator "spinner" of window "main" to start
			display panel window "confirm" attached to window "main"
		else
			startStopButton()
		end if
	else if the name of theObject is equal to "yesstart" then
		close panel window "confirm"
		startStopButton()
		
	else if the name of theObject is equal to "updatedebug" then
		updatedebug()
	else if the name of theObject is equal to "closeit" then
		tell progress indicator "spinner" of window "main" to stop
		close panel window of theObject
		set the visible of image view "picerr" of tab view item "firewall" of tab view "main" of window "main" to false as boolean
		set the visible of image view "picmid" of tab view item "firewall" of tab view "main" of window "main" to false as boolean
		try
			set the contents of text field "bannerfirewall" of tab view item "firewall" of tab view "main" of window "main" to oldStatus
		end try
	else if the name of theObject is equal to "nostart" then
		close panel window of theObject
		tell progress indicator "spinner" of window "main" to stop
		set the visible of image view "picerr" of tab view item "firewall" of tab view "main" of window "main" to false as boolean
		set the visible of image view "picmid" of tab view item "firewall" of tab view "main" of window "main" to false as boolean
		set the contents of text field "bannerfirewall" of tab view item "firewall" of tab view "main" of window "main" to ""
		
	else if the name of theObject is equal to "confirmpreset" then
		set the contents of text field "currentpreset" of tab view item "firewall" of tab view "main" of window "main" to ""
		enablePreset()
		
	end if
end clicked

on startStopButton()
	if the title of button "startstop" of tab view item "firewall" of tab view "main" of window "main" is equal to "Start" then
		set the contents of text field "bannerfirewall" of tab view item "firewall" of tab view "main" of window "main" to "Starting firewall..."
		applyRuleset()
		checkPFstatus()
		checkStartupScripts()
	else
		PFstop()
		checkPFstatus()
	end if
end startStopButton






on choose menu item theObject
	if the name of theObject is equal to "stoppf" then
		set pfstatus to do shell script "pfctl -si |grep \"Status:\" |awk '{print $2}'" with administrator privileges
		if pfstatus is equal to "Enabled" then
			try
				do shell script "pfctl -d" with administrator privileges
			end try
		end if
	else if the name of theObject is equal to "pfstop" then
		PFstop()
	else if the name of theObject is equal to "apply" then
		PFapply()
	else if the name of theObject is equal to "restartpf" then
		PFrestart()
	else if the name of theObject is equal to "installscripts" then
		installScripts()
		displayDialogOK("Boot scripts installed.")
	else if the name of theObject is equal to "removescripts" then
		uninstallScripts()
	else if the name of theObject is equal to "killall" then
		do shell script "pfctl -k 0.0.0.0/0 -k 0.0.0.0/0" with administrator privileges
		displayDialogOK("All connections closed.")
	else if the name of theObject is equal to "export" then
		
	else if the name of theObject is equal to "import" then
		
	else if the name of theObject is equal to "about" then
		show window "about"
		
	end if
end choose menu item


----------------------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------
--           APPLY RULES
----------------------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------

on applyRuleset()
	-- check for /Library/PFLists directory
	try
		do shell script "ls /Library/PFLists"
	on error
		--displayDialogOK("Creating /Library/PFLists directory to store PF configuration...")
		do shell script "mkdir /Library/PFLists" with administrator privileges
		do shell script "chmod -R ug+rw /Library/PFLists" with administrator privileges
		-- change group to 'admin'
		do shell script "chgrp admin /Library/PFLists" with administrator privileges
	end try
	getPath()
	checkOSXversion()
	-- fix privileges
	do shell script "chmod -R ug+rw /Library/PFLists" with administrator privileges
	-- INSTALL ET TABLES IF NEEDED
	try
		do shell script "ls /Library/PFLists/pflists.ettable"
	on error
		-- ettable not found, copy empty emerging threats PF table
		do shell script "cp \"" & binaryPath & "pflists.ettable\" /Library/PFLists/pflists.ettable"
	end try
	
	--- set banner
	if the contents of text field "bannerfirewall" of tab view item "firewall" of tab view "main" of window "main" is not equal to "Starting firewall..." then
		set the contents of text field "bannerfirewall" of tab view item "firewall" of tab view "main" of window "main" to "Applying changes..."
	end if
	
	-- APPLY RULESET
	createTables() -- build pf table for each address group
	emergingThreats() -- build emerging threats pf table
	createRules() -- add rules to pflists.conf
	checkOptions() -- set comapple and et options
	tell progress indicator "spinner" of window "main" to stop
	checkPFstatus() -- update on-off button state
	-- fix privileges
	do shell script "chmod -R ug+rw /Library/PFLists" with administrator privileges
end applyRuleset
----------------------------------------------------------------------------------------------------------------------------------------------



----------------------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------
--  CREATE PF TABLES
----------------------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------

-- create PF tables (a new pf table for each directory in LISTS dir)
on createTables()
	try
		set the contents of text field "bannerfirewall" of tab view item "firewall" of tab view "main" of window "main" to "Building tables..."
		-- count lists (count directories in LISTS directory)
		set listsCount to do shell script "ls " & appdir & "default/LISTS/" & " |wc -l |bc"
		-- get lists list (used with parser in 'repeat')
		set listsList to do shell script "ls -l " & appdir & "default/LISTS/" & " |awk '{print $9}' |sed '/./,$!d'"
		-- pflists.tables conf file
		try
			do shell script "rm -f " & tablesFile & ""
		end try
		do shell script "touch " & tablesFile & ""
		-- loop
		repeat with loopCounter from 1 to listsCount
			-- lists all directories (each dir is a PFLists list)
			set listName to paragraph loopCounter of listsList
			set theTable to "table <" & listName & "> {"
			set theTable to theTable & ((do shell script "cat \"" & appdir & "default/LISTS/" & listName & "/content\" | sed 's/@//' | tr \"-\" \" \" | sed 's/ //' | tr \"\\n\" \" \"") & "} persist")
			do shell script "echo \"" & theTable & "\" >> " & tablesFile & ""
		end repeat
	on error errorTxt
		displayDialogOK("tables error: " & errorTxt)
	end try
	set the contents of text field "bannerfirewall" of tab view item "firewall" of tab view "main" of window "main" to "Emerging Threats..."
end createTables

----------------------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------
-- CREATE PF RULES
----------------------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------

on createRules()
	set the contents of text field "bannerfirewall" of tab view item "firewall" of tab view "main" of window "main" to "Building rules..."
	try
		-----------------------------------------
		-----------------------------------------
		-----------------------------------------
		-- INBOUND RULES
		-----------------------------------------
		-----------------------------------------
		-----------------------------------------
		-- count lists (count directories in LISTS directory)
		set listsCount to do shell script "ls " & appdir & "default/LISTS/" & " |wc -l |bc"
		-- get lists list (used with parser in 'repeat')
		set listsList to do shell script "ls -l " & appdir & "default/LISTS/" & " |awk '{print $9}' |sed '/./,$!d'"
		-- pflists.conf conf file
		try
			do shell script "rm -f " & confFile & ""
		end try
		do shell script "touch " & confFile & ""
		
		--------------------------------------------------------------------------------------------------------------------------------------------------------------------
		--					COPY CORRECT VERSION OF PFLISTS.CONF FILE (enable OR disable anchor com.apple)
		--------------------------------------------------------------------------------------------------------------------------------------------------------------------
		
		set currentOption to do shell script "cat \"" & appdir & "default/OPTIONS/options\" |grep \"comapple\" |cut -d \":\" -f 2"
		--
		if currentOption is equal to "1" then -- enable com.apple anchor
			if macosxVersion is not equal to "10.7" then
				do shell script "cp \"" & binaryPath & "pflists.conf\" " & confFile & ""
			else
				do shell script "cp \"" & binaryPath & "pflists10.7.conf\" " & confFile & ""
			end if
		else
			-- no apple.com anchor
			if macosxVersion is not equal to "10.7" then
				do shell script "cp \"" & binaryPath & "pflists-noapple.conf\" " & confFile & ""
			else
				do shell script "cp \"" & binaryPath & "pflists-noapple10.7.conf\" " & confFile & ""
			end if
			--
		end if
		
		--------------------------------------------------------------------------------------------------------------------------------------------------------------------
		--					DUMMYNET BANDWIDTH
		--------------------------------------------------------------------------------------------------------------------------------------------------------------------
		
		-- DUMMYNET ANCHOR
		do shell script "touch /Library/PFLists/pflists.bandwidth" -- touch anchor
		do shell script "cp \"" & binaryPath & "dummynet.sh\" /Library/PFLists/dummynet.sh" -- script for ipfw pipe
		--
		try
			set dummyBW to do shell script "cat \"" & appdir & "default/OPTIONS/dummynet\""
			if dummyBW is equal to "" then
				-- no dummynet
				-- build anchor
				do shell script "echo \"\" > /Library/PFLists/pflists.bandwidth"
				--displayDialogOK("no dummynet")
			else
				--displayDialogOK("dummynet")
				-- get my network address
				set myNetAddr to getMyNetAddress()
				-- build anchor
				do shell script "echo \"no dummynet quick on lo0 all\" > /Library/PFLists/pflists.bandwidth"
				do shell script "echo \"dummynet in quick inet from !" & myNetAddr & " to any allow-opts pipe 12345\" >> /Library/PFLists/pflists.bandwidth"
				do shell script "echo \"dummynet out quick inet from any to !" & myNetAddr & " allow-opts pipe 12345\"  >> /Library/PFLists/pflists.bandwidth"
				-- set ipfw pipe file
				do shell script "echo \"ipfw pipe 12345 config bw " & dummyBW & "Kbit/s\" > /Library/PFLists/dummynet.sh"
				-- set ipfw pipe in runtime
				do shell script "ipfw pipe 12345 config bw " & dummyBW & "Kbit/s" with administrator privileges
			end if
		on error
			do shell script "touch \"" & appdir & "default/OPTIONS/dummynet\""
		end try
		
		
		
		
		
		
		
		--------------------------------------------------------------------------------------------------------------------------------------------------------------------
		--					COPY GENERIC IPV6 RULES 
		--------------------------------------------------------------------------------------------------------------------------------------------------------------------
		
		do shell script "cp \"" & binaryPath & "icefloor.genericipv6\" /Library/PFLists/pflists.genericipv6"
		
		--------------------------------------------------------------------------------------------------------------------------------------------------------------------
		--					inbound LOOP
		--------------------------------------------------------------------------------------------------------------------------------------------------------------------
		
		-- for each group we look at services and we take all protocols
		-- we must know how many protocols type is prensent for each group
		repeat with loopCounter from 1 to listsCount
			set the contents of text field "bannerfirewall" of tab view item "firewall" of tab view "main" of window "main" to "Building rules " & loopCounter & "/" & listsCount
			set listName to paragraph loopCounter of listsList
			-- list all protocols for current list
			set protocolsList to do shell script "cat \"" & appdir & "default/LISTS/" & listName & "/services\" | cut -d \"#\" -f 3 |sort |uniq"
			-- count all protocols for current list
			set protocolsCount to count paragraph of protocolsList
			-- loop (for each protocol in each group) excluding list "_outbound"
			-- then, for each protocol, we parse the services file to get all ports to be opened for 
			-- this specific protocol
			if listName is not equal to "_outbound" then
				--displayDialogOK("building rules for group " & listName)
				repeat with protoCounter from 1 to protocolsCount
					set protocolName to paragraph protoCounter of protocolsList
					-- get all ports for this proto
					--displayDialogOK("actual protocol parsed:" & protocolName)
					set protocolsPorts to do shell script "cat \"" & appdir & "default/LISTS/" & listName & "/services\" | grep \"#" & protocolName & "\" |cut -d \"#\" -f4 | tr \"\\n\" \" \""
					set theRule to "pass in proto " & protocolName & " from <" & listName & "> to any"
					--
					if protocolsPorts is equal to " " or protocolsPorts is equal to "icmp" or protocolsPorts is equal to "igmp" or protocolsPorts is equal to "esp" or protocolsPorts is equal to "gre" then -- selected protocol does not accept ports
						-- no ports
					else
						set theRule to theRule & " port { " & protocolsPorts & " } "
					end if
					-- set flags
					if protocolName is equal to "tcp" then
						set theRule to theRule & " flags S/SA keep state "
					end if
					if protocolName is equal to "udp" then
						set theRule to theRule & " keep state "
					end if
					if protocolName is equal to "icmp" then
						set theRule to theRule & " keep state "
					end if
					if protocolName is equal to "igmp" then
						set theRule to theRule & " keep state "
					end if
					if protocolName is equal to "esp" then
						set theRule to theRule & " keep state "
					end if
					if protocolName is equal to "gre" then
						set theRule to theRule & " keep state "
					end if
					if protocolName is equal to "all" then
						set theRule to theRule & " flags S/SA keep state "
					end if
					-- purge 'all' protocol
					if protocolName is equal to "all" then
						set theRule to do shell script "echo \"" & theRule & "\" |sed 's/proto all/proto {tcp udp}/'"
					end if
					--displayDialogOK(theRule)
					-- write rule to configuration file
					do shell script "echo \"" & theRule & "\" >> " & confFile & ""
				end repeat
			end if
		end repeat
		--------------------------------------------------------------------------------------------------------------------------------------------------------------------
		
	end try
	
	-----------------------------------------
	-----------------------------------------
	-----------------------------------------
	-- OUTBOUND RULES
	-----------------------------------------
	-----------------------------------------
	-----------------------------------------
	
	set the contents of text field "bannerfirewall" of tab view item "firewall" of tab view "main" of window "main" to "Outbound rules... "
	
	do shell script "echo \"### OUTBOUND RULES SET BY PFLISTS.APP ###\" >> " & confFile & ""
	try
		set listName to "_outbound"
		-- list all protocols for current list
		set protocolsList to do shell script "cat \"" & appdir & "default/LISTS/" & listName & "/services\" | cut -d \"#\" -f 3 |sort |uniq"
		-- count all protocols for current list
		set protocolsCount to count paragraph of protocolsList
		repeat with protoCounter from 1 to protocolsCount
			set protocolName to paragraph protoCounter of protocolsList
			-- get all ports for this proto
			--displayDialogOK("actual protocol parsed:" & protocolName)
			set protocolsPorts to do shell script "cat \"" & appdir & "default/LISTS/" & listName & "/services\" | grep \"#" & protocolName & "\" |cut -d \"#\" -f4 | tr \"\\n\" \" \""
			set theRule to "pass out proto " & protocolName & " from any to <" & listName & ">"
			
			if protocolsPorts is equal to " " or protocolsPorts is equal to "icmp" or protocolsPorts is equal to "igmp" or protocolsPorts is equal to "esp" or protocolsPorts is equal to "gre" then -- selected protocol does not accept ports
				-- no ports
			else
				set theRule to theRule & " port { " & protocolsPorts & " } "
			end if
			-- set flags
			if protocolName is equal to "tcp" then
				set theRule to theRule & " flags S/SA keep state "
			end if
			if protocolName is equal to "udp" then
				set theRule to theRule & " keep state "
			end if
			if protocolName is equal to "icmp" then
				set theRule to theRule & " keep state "
			end if
			if protocolName is equal to "igmp" then
				set theRule to theRule & " keep state "
			end if
			if protocolName is equal to "esp" then
				set theRule to theRule & " keep state "
			end if
			if protocolName is equal to "gre" then
				set theRule to theRule & " keep state "
			end if
			if protocolName is equal to "all" then
				set theRule to theRule & " flags S/SA keep state "
			end if
			-- purge 'all' protocol
			if protocolName is equal to "all" then
				set theRule to do shell script "echo \"" & theRule & "\" |sed 's/proto all/proto {tcp udp}/'"
			end if
			-- 
			-- write rule to configuration file
			do shell script "echo \"" & theRule & "\" >> " & confFile & ""
		end repeat
		
	end try
	
	
	-- finally apply pf rules
	PFapply()
end createRules



----------------------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------
-- DO APPLY PF CONF
--------------------------------------------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------------------------------------------
on PFapply()
	set pfstatus to do shell script "pfctl -si |grep \"Status:\" |awk '{print $2}'" with administrator privileges
	if pfstatus is equal to "Enabled" then
		-- pf already enabled
	else
		try
			do shell script "pfctl -E" with administrator privileges -- enable pf
		end try
	end if
	try
		do shell script "pfctl -f \"" & confFile & "\"" with administrator privileges
		set the visible of image view "picerr" of tab view item "firewall" of tab view "main" of window "main" to false as boolean
		set the visible of image view "picmid" of tab view item "firewall" of tab view "main" of window "main" to false as boolean
	on error errorText
		set the visible of image view "picerr" of tab view item "firewall" of tab view "main" of window "main" to true as boolean
		set the visible of image view "picmid" of tab view item "firewall" of tab view "main" of window "main" to false as boolean
		--
		set the visible of image view "picoff" of tab view item "firewall" of tab view "main" of window "main" to true as boolean
		set the visible of image view "picon" of tab view item "firewall" of tab view "main" of window "main" to false as boolean
		tell progress indicator "spinner" of window "main" to stop
		showError(errorText)
	end try
end PFapply


--------------------------------------------------------------------------------------------------------------------------------------------------------------------
on PFrestart()
	set pfstatus to do shell script "pfctl -si |grep \"Status:\" |awk '{print $2}'" with administrator privileges
	if pfstatus is equal to "Enabled" then
		try
			do shell script "pfctl -d" with administrator privileges
		end try
		try
			do shell script "pfctl -Ef \"" & confFile & "\"" with administrator privileges
		on error errorText
			displayDialogOK("PF ERROR: " & errorText)
		end try
	end if
end PFrestart

--------------------------------------------------------------------------------------------------------------------------------------------------------------------
on PFstop()
	set pfstatus to do shell script "pfctl -si |grep \"Status:\" |awk '{print $2}'" with administrator privileges
	if pfstatus is equal to "Enabled" then
		do shell script "pfctl -d" with administrator privileges
	end if
end PFstop



----------------------------------------------------------------------------------------------------------------------------------------------
-- ENABLE SELECTED PRESET
----------------------------------------------------------------------------------------------------------------------------------------------

on enablePreset()
	set the contents of text field "bannerfirewall" of tab view item "firewall" of tab view "main" of window "main" to "Enabling preset..."
	set presetName to the contents of text field "presetname" of window "confirmpreset"
	close panel window "confirmpreset"
	do shell script "rm -rf \"" & appdir & "default\""
	do shell script "cp -R \"" & appdir & "presets/" & presetName & "\" \"" & appdir & "default\""
	applyRuleset()
	set the content of table view "lists" of scroll view "lists" of tab view item "firewall" of tab view "main" of window "main" to {}
	display panel window "donepreset" attached to window "main"
end enablePreset



----------------------------------------------------------------------------------------------------------------------------------------------
-- EMERGING THREATS MANAGEMENT
----------------------------------------------------------------------------------------------------------------------------------------------

on emergingThreats()
	set currentDate to do shell script "date"
	set emergingOption to do shell script "cat \"" & appdir & "default/OPTIONS/options\" |grep \"emerging\" |cut -d \":\" -f 2"
	--
	--
	if emergingOption is equal to "1" then -- et is enabled
		try
			display panel window "emerging" attached to window "main"
			set the contents of text field "bannerfirewall" of tab view item "firewall" of tab view "main" of window "main" to "Downloading..."
			do shell script "curl -m 10 " & etURL & " > /tmp/tmp_eturl" -- download ET list and write to temporary file
			do shell script "cat /tmp/tmp_eturl |grep -o '[0-9]\\{1,3\\}\\.[0-9]\\{1,3\\}\\.[0-9]\\{1,3\\}\\.[0-9]\\{1,3\\}'  | tr \"\\n\" \" \" | tr \",\" \" \" > /tmp/tmp_etlist" -- create table
			do shell script "rm -f /tmp/tmp_eturl"
			-- build pflists.ettable file
			do shell script "echo \"### PFLists Emerging Threats table\" > /Library/PFLists/pflists.ettable"
			do shell script "echo \"### Last update: " & currentDate & "\" >> /Library/PFLists/pflists.ettable"
			do shell script "echo \"###\" >> /Library/PFLists/pflists.ettable"
			do shell script "printf \"%s\" \"table <emergingthreats> { \" >> /Library/PFLists/pflists.ettable"
			do shell script "cat /tmp/tmp_etlist >> /Library/PFLists/pflists.ettable"
			do shell script "echo \" } persist\" >> /Library/PFLists/pflists.ettable"
			--
			-- copy et autoupdate scripts
			installETscripts()
		on error
			set the contents of text field "bannerfirewall" of tab view item "firewall" of tab view "main" of window "main" to "Not available!"
			-- on error copy empty emerging threats PF table
			do shell script "cp \"" & binaryPath & "pflists.ettable\" /Library/PFLists/pflists.ettable"
		end try
	else -- et is disabled
		uninstallETscripts()
		-- copy empty emerging threats PF table
		do shell script "cp \"" & binaryPath & "pflists.ettable\" /Library/PFLists/pflists.ettable"
	end if
	close panel window "emerging"
end emergingThreats






-- check PF status
on checkPFstatus()
	try
		set pfstatus to do shell script "pfctl -si |grep \"Status:\" |awk '{print $2}'" with administrator privileges
		if pfstatus is equal to "Enabled" then
			set the contents of text field "bannerfirewall" of tab view item "firewall" of tab view "main" of window "main" to ""
			enableStop()
		else if pfstatus is equal to "Disabled" then
			set the contents of text field "bannerfirewall" of tab view item "firewall" of tab view "main" of window "main" to ""
			enableStart()
		end if
	end try
end checkPFstatus


--
on checkStartupScripts()
	try
		do shell script "ls /Library/LaunchDaemons/com.hanynet.icefloor.plist"
		do shell script "ls /etc/icefloor.sh"
	on error
		set question to display dialog "Do you want to install PFLists Boot Scripts to load PF rules at system boot?" buttons {"Yes", "No"} default button 1
		set answer to button returned of question
		if answer is equal to "Yes" then
			installScripts()
		end if
	end try
	--
	tell progress indicator "spinner" of window "main" to stop
end checkStartupScripts


-------------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------------------
-- START BUTTON
on enableStart()
	set the visible of button "apply" of tab view item "firewall" of tab view "main" of window "main" to false as boolean
	set the visible of button "enablep" of tab view item "presets" of tab view "main" of window "main" to false as boolean
	set the title of button "startstop" of tab view item "firewall" of tab view "main" of window "main" to "Start"
	set the visible of image view "picon" of tab view item "firewall" of tab view "main" of window "main" to false as boolean
	set the visible of image view "picoff" of tab view item "firewall" of tab view "main" of window "main" to true as boolean
	set the visible of image view "picerr" of tab view item "firewall" of tab view "main" of window "main" to false as boolean
	set the visible of image view "picmid" of tab view item "firewall" of tab view "main" of window "main" to false as boolean
end enableStart
--
on enableStop()
	set the visible of button "apply" of tab view item "firewall" of tab view "main" of window "main" to true as boolean
	set the visible of button "enablep" of tab view item "presets" of tab view "main" of window "main" to true as boolean
	set the title of button "startstop" of tab view item "firewall" of tab view "main" of window "main" to "Stop"
	set the visible of image view "picon" of tab view item "firewall" of tab view "main" of window "main" to true as boolean
	set the visible of image view "picoff" of tab view item "firewall" of tab view "main" of window "main" to false as boolean
	set the visible of image view "picerr" of tab view item "firewall" of tab view "main" of window "main" to false as boolean
	set the visible of image view "picmid" of tab view item "firewall" of tab view "main" of window "main" to false as boolean
end enableStop

----------------------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------
--				CHECK OPTIONS
----------------------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------


-- CHECK OPTIONS (read pflists options file and set interface accordingly)

on checkOptions()
	-- COM.APPLE option
	try
		set comAppleOption to do shell script "cat \"" & appdir & "default/OPTIONS/options\" |grep \"comapple\" |cut -d \":\" -f 2"
		if comAppleOption is equal to "1" then
			set the content of button "comapple" of tab view item "options" of tab view "main" of window "main" to true as boolean
		else
			set the content of button "comapple" of tab view item "options" of tab view "main" of window "main" to false as boolean
		end if
	end try
	-- DUMMYNET BANDWIDTH option
	try
		set dummynetOption to ""
		set dummynetOption to do shell script "cat \"" & appdir & "default/OPTIONS/dummynet\""
		if dummynetOption is equal to "" then
			set the content of button "dummynet" of tab view item "options" of tab view "main" of window "main" to false as boolean
		else
			set the content of button "dummynet" of tab view item "options" of tab view "main" of window "main" to true as boolean
			set the content of text field "dummynet" of tab view item "options" of tab view "main" of window "main" to dummynetOption
		end if
	end try
	-- EMERGING THREATS option
	try
		set emergingOption to do shell script "cat \"" & appdir & "default/OPTIONS/options\" |grep \"emerging\" |cut -d \":\" -f 2"
		if emergingOption is equal to "1" then
			set the content of button "emerging" of tab view item "options" of tab view "main" of window "main" to true as boolean
		else
			set the content of button "emerging" of tab view item "options" of tab view "main" of window "main" to false as boolean
		end if
	end try
	-- LOGGING EXCEPTIONS 
	try
		set the contents of text field "inboundtcp" of window "exceptions" to do shell script "cat \"" & appdir & "default/OPTIONS/exceptions\" |grep \"inboundtcp\" |cut -d \"#\" -f 2"
		set the contents of text field "outboundtcp" of window "exceptions" to do shell script "cat \"" & appdir & "default/OPTIONS/exceptions\" |grep \"outboundtcp\" |cut -d \"#\" -f 2"
		set the contents of text field "inboundudp" of window "exceptions" to do shell script "cat \"" & appdir & "default/OPTIONS/exceptions\" |grep \"inboundudp\" |cut -d \"#\" -f 2"
		set the contents of text field "outboundudp" of window "exceptions" to do shell script "cat \"" & appdir & "default/OPTIONS/exceptions\" |grep \"outboundudp\" |cut -d \"#\" -f 2"
		set the contents of text field "ignoredips" of window "exceptions" to do shell script "cat \"" & appdir & "default/OPTIONS/exceptions\" |grep \"ignoredips\" |cut -d \"#\" -f 2"
		set the contents of button "ignoredicmp" of window "exceptions" to (do shell script "cat \"" & appdir & "default/OPTIONS/exceptions\" |grep \"ignoredicmp\" |cut -d \"#\" -f 2") as boolean
		set the contents of button "ignoredicmp6" of window "exceptions" to (do shell script "cat \"" & appdir & "default/OPTIONS/exceptions\" |grep \"ignored6icmp\" |cut -d \"#\" -f 2") as boolean
		set the contents of button "ignoredigmp" of window "exceptions" to (do shell script "cat \"" & appdir & "default/OPTIONS/exceptions\" |grep \"ignoredigmp\" |cut -d \"#\" -f 2") as boolean
		set the contents of button "ignoredipv6" of window "exceptions" to (do shell script "cat \"" & appdir & "default/OPTIONS/exceptions\" |grep \"ignoredipv6\" |cut -d \"#\" -f 2") as boolean
	end try
	-- PORT KNOCKING
	try
		set knockingOption to do shell script "cat \"" & appdir & "default/OPTIONS/options\" |grep \"knocking\" |cut -d \":\" -f 2"
		if knockingOption is equal to "1" then
			set the content of button "knocking" of tab view item "knocking" of tab view "main" of window "main" to true as boolean
		else
			set the content of button "knocking" of tab view item "knocking" of tab view "main" of window "main" to false as boolean
		end if
	end try
end checkOptions
----------------------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------






----------------------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------
--   MISC
----------------------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------

-- get app bundle path
on getPath()
	try
		-- prendo path del bundle come stringa per poterla modificare
		set myPath to path to me as string
		-- aggiungo parte mancante al path dei binari eseguibili
		set macPath to (myPath & "Contents:Resources:")
		-- converto in formato POSIX il path dei binari esebuibili
		set binaryPath to POSIX path of macPath
	end try
end getPath

---------- DIALOG HANDLERS
on displayDialogOK(theMsg)
	try
		set theDialog to display dialog (theMsg) buttons {"OK"} default button 1 with icon 1 -- attached to window (get first responder)
		set dialogReturn to "ok"
	on error
		set dialogReturn to "cancel"
	end try
end displayDialogOK

on installScripts()
	try
		do shell script "cp \"" & binaryPath & "icefloor.sh\" /etc/icefloor.sh" with administrator privileges
		-- rendi script eseguibile
		do shell script "chmod +x /etc/icefloor.sh" with administrator privileges
		-- copia file plist per avvio automatico di routes.sh con launchd in /Library
		do shell script "cp \"" & binaryPath & "com.hanynet.icefloor.plist\" /Library/LaunchDaemons/com.hanynet.icefloor.plist" with administrator privileges
		-- cambia id e gid del file plist 
		do shell script "chown root:wheel /Library/LaunchDaemons/com.hanynet.icefloor.plist" with administrator privileges
		do shell script "launchctl load -w /Library/LaunchDaemons/com.hanynet.icefloor.plist" with administrator privileges
		-- xattr
		do shell script "xattr -c /Library/LaunchDaemons/com.hanynet.icefloor.plist" with administrator privileges
	on error errorText
		displayDialogOK("ERROR: " & errorText)
	end try
	checkSysLogConf() -- ENABLE LOGGING
end installScripts

on uninstallScripts()
	try
		do shell script "rm -f /etc/icefloor.sh" with administrator privileges
		do shell script "rm -f /Library/LaunchDaemons/com.hanynet.icefloor.plist" with administrator privileges
	on error errorText
		displayDialogOK("ERROR: " & errorText)
	end try
	displayDialogOK("Boot scripts uninstalled.")
end uninstallScripts

--- ET SCRIPTS

on installETscripts() -- controllare se esistono gia e NON installare!!!!!!!!!
	try
		do shell script "ls /Library/LaunchDaemons/com.hanynet.pflists.emergingthreats.plist"
	on error
		-- script does not exist, install it
		try
			do shell script "cp \"" & binaryPath & "pflists_emergingthreats.sh\" /Library/PFLists/pflists_emergingthreats.sh"
			do shell script "cp \"" & binaryPath & "com.hanynet.pflists.emergingthreats.plist\" /Library/LaunchDaemons/com.hanynet.pflists.emergingthreats.plist" with administrator privileges
			-- cambia id e gid del file plist 
			do shell script "chown root:wheel /Library/LaunchDaemons/com.hanynet.pflists.emergingthreats.plist" with administrator privileges
			-- xattr
			do shell script "xattr -c /Library/LaunchDaemons/com.hanynet.pflists.emergingthreats.plist" with administrator privileges
			do shell script "launchctl load -w /Library/LaunchDaemons/com.hanynet.pflists.emergingthreats.plist" with administrator privileges
		end try
		try
			do shell script "launchctl load -w /Library/LaunchDaemons/com.hanynet.pflists.emergingthreats.plist" with administrator privileges
		end try
	end try
end installETscripts

on uninstallETscripts() -- controllare se non esistono gia e NON installare!!!!!!!!!
	set thescriptExist to ""
	try
		set thescriptExist to do shell script "ls /Library/LaunchDaemons/com.hanynet.pflists.emergingthreats.plist"
	end try
	if thescriptExist is equal to "" then
		--displayDialogOK("ET script does not exist, skip uninstall...")
	else
		try
			do shell script "launchctl load -w /Library/LaunchDaemons/com.hanynet.pflists.emergingthreats.plist" with administrator privileges
		end try
		try
			do shell script "rm -f /Library/LaunchDaemons/com.hanynet.pflists.emergingthreats.plist" with administrator privileges
		end try
		try
			do shell script "rm -f /Library/PFLists/pflists_emergingthreats.sh"
		end try
	end if
end uninstallETscripts


-- CHECK IF SYSLOG.CONF IS ENABLED FOR PF LOGGING... IF NOT THEN UPDATE
-- this file must be added to /etc/syslog.conf :
-- local2.*                                                /var/log/pffirewall.log
on checkSysLogConf()
	---------------- PASS PF LOGS TO SYSLOG AND CREATE LOG FILE -----------------------------
	set checkFile to ""
	try
		set checkFile to do shell script "cat /etc/syslog.conf | grep \"/var/log/pffirewall.log\""
	end try
	if checkFile is equal to "" then
		--displayDialogOK("We must update /etc/syslog.conf in order to activate PF logs...")
		do shell script "echo \"local2.*                                                /var/log/pffirewall.log\" >> /etc/syslog.conf" with administrator privileges
	else
		--displayDialogOK("/etc/syslog.conf is ok.")
	end if
	-------------------- ROTATE LOGS -------------------
	set checkFile to ""
	try
		set checkFile to do shell script "cat /etc/newsyslog.conf | grep \"/var/log/pffirewall.log\""
	end try
	if checkFile is equal to "" then
		-- rotate logs (thanks to S.Kolins and A.Lauzon)
		--displayDialogOK("We must update /etc/newsyslog.conf in order to rotate PF logs... Logging will be enabled at next system boot.")
		do shell script "echo \"/var/log/pffirewall.log			640  20	   5000	*     J\" >> /etc/newsyslog.conf" with administrator privileges
	else
		--displayDialogOK("/etc/newsyslog.conf is ok.")
	end if
end checkSysLogConf
-------------------

-- GET MY NETWORK ADDRESS

on getMyNetAddress()
	getPath()
	set mainInterface to ""
	set mainInterface to do shell script "route -n get 0.0.0.0 |grep interface |awk '/interface: / {print $2}'"
	
	if mainInterface contains "not in table" or mainInterface contains "routing socket" or mainInterface is equal to "" then
		debugError()
	end if
	
	set LocalIPaddress to do shell script "ifconfig " & mainInterface & " |grep \"inet \" |awk '{print $2}' |sed -n 1p"
	set LocalHEXMask to do shell script "ifconfig " & mainInterface & " |grep \"inet \" |awk '{print $4}' |sed -n 1p"
	set LocalMask to do shell script "sh \"" & binaryPath & "hex2dotdec\" " & LocalHEXMask
	set LocalNetworkAddress to do shell script "sh \"" & binaryPath & "mask\" " & LocalIPaddress & " " & LocalMask
	--
	return LocalNetworkAddress
end getMyNetAddress


on uninstallEverything()
	set question to display dialog "Are you sure? All PFLists preferences will be lost, firewall will be reset to factory default and PFLists will quit. Proceed with uninstall? " buttons {"Yes", "No"} default button 1
	set answer to button returned of question
	if answer is equal to "yes" then
		-- stop pf
		set pfstatus to do shell script "pfctl -si |grep \"Status:\" |awk '{print $2}'" with administrator privileges
		if pfstatus is equal to "Enabled" then
			try
				do shell script "pfctl -d" with administrator privileges
			end try
		end if
		-- remove pflists preferences directory
		try
			do shell script "rm -rf \"" & appdir & "\"" with administrator privileges
		end try
		-- remove pflists configuration 
		try
			do shell script "rm -rf /Library/PFLists" with administrator privileges
		end try
		try
			-- remove startup scripts
			do shell script "rm -f /etc/icefloor.sh" with administrator privileges
			do shell script "rm -f /Library/LaunchDaemons/com.hanynet.icefloor.plist" with administrator privileges
		end try
		try
			-- reset logging
			-- reset syslog and newsyslog
			
			do shell script "cat /etc/newsyslog.conf |grep -v /var/log/pffirewall.log > /tmp/icefloor_newsyslog.conf" with administrator privileges
			do shell script "cat /etc/syslog.conf |grep -v /var/log/pffirewall.log > /tmp/icefloor_syslog.conf" with administrator privileges
			do shell script "cp /tmp/icefloor_newsyslog.conf /etc/newsyslog.conf" with administrator privileges
			do shell script "cp /tmp/icefloor_syslog.conf /etc/syslog.conf" with administrator privileges
			do shell script "rm /tmp/icefloor_newsyslog.conf" with administrator privileges
			do shell script "rm /tmp/icefloor_syslog.conf" with administrator privileges
			
		end try
		try
			-- uninstall emerging threats autoupdate scripts
			uninstallETscripts()
		end try
		quit
		
	end if
	
end uninstallEverything

-- MODIFY PRESET 

on doModifyPreset()
	try
		do shell script "echo \"custom#\" > \"" & appdir & "default/OPTIONS/description\""
	end try
	set the contents of text field "currentpreset" of tab view item "firewall" of tab view "main" of window "main" to "Current preset: custom"
end doModifyPreset

-------------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------------------------------------
-- error handling

on showError(theError)
	try
		set theError to do shell script "echo \"" & theError & "\" |sed 's/dyld: DYLD_ environment variables being ignored because main executable (\\/usr\\/libexec\\/security_authtrampoline) is setuid or setgid//' "
		set theError to do shell script "echo \"" & theError & "\" |sed 's/No ALTQ support in kernel//' "
		set theError to do shell script "echo \"" & theError & "\" |sed 's/ALTQ related functions disabled//' "
	end try
	set the contents of text view "error" of scroll view "error" of window "error" to theError
	display panel window "error" attached to window "main"
end showError


-- CHECK OS X VERSION
on checkOSXversion()
	try
		set osVersion to do shell script "uname -r | cut -d \".\" -f1"
		if osVersion is equal to "8" then
			displayDialogOK("Sorry, Mac OS X 10.4 is not supported. Please use WaterRoof or NoobProof")
			quit
		else if osVersion is equal to "9" then
			displayDialogOK("Sorry, Mac OS X 10.5 is not supported. Please use WaterRoof or NoobProof")
			quit
		else if osVersion is equal to "10" then
			displayDialogOK("Sorry, Mac OS X 10.6 is not supported. Please use WaterRoof or NoobProof")
			quit
		else if osVersion is equal to "11" then
			set macosxVersion to "10.7"
			
		else if osVersion is equal to "12" then
			set macosxVersion to "10.8"
			
		else if osVersion is equal to "13" then
			set macosxVersion to "10.9"
			
		else if osVersion is equal to "14" then
			set macosxVersion to "10.10"
			
		end if
		-- check for Mac OS X Server
		set osVer to do shell script "sw_vers |grep ProductName: |cut -f2"
		if osVer is not equal to "Mac OS X" then
			--displayDialogOK("OS X Server untested, use at your own risk.")
			-- quit
		end if
	end try
end checkOSXversion






























